#!/bin/bash

KML_API_DIR="$(python3 -c 'import kml.api; import os; print(os.path.dirname(kml.api.__file__))')"
# KML_SWEEPER_DIR="$(python3 -c 'import kml.puppeteer; import os; print(os.path.dirname(kml.puppeteer.__file__))')"

echo $(date +"%F %T")",000 INFO  (KML) startup - Starting KML server"
echo $(date +"%F %T")",000 INFO  (KML) startup - Module path "$KML_API_DIR
# echo $KML_SWEEPER_DIR

MODULE="kml.api.api"
CONFIG_FILE="${KML_API_DIR}/kml.ini"

if [ -z "${KML_API_DIR}" ] || [ ! -f "${CONFIG_FILE}" ]; then
    echo $(date +"%F %T")",000 INFO  (KML) startup - ERROR: Unable to locate the kml.ini file at ${CONFIG_FILE}" >&2
    exit 1
fi

# Load config parameters
conn_str=$(awk -F "=" '/api_connection/ {print $2}' $CONFIG_FILE | sed -E 's~^[ \t]+~~g' | sed -E 's~[ \t]+$~~g')
db_str=$(awk -F "=" '/db_connection/ {print $2}' $CONFIG_FILE | sed -E 's~^[ \t]+~~g' | sed -E 's~[ \t]+$~~g')
log_level=$(awk -F "=" '/log_level/ {print $2}' $CONFIG_FILE | sed -E 's~^[ \t]+~~g' | sed -E 's~[ \t]+$~~g')
debug_mode=$(awk -F "=" '/server_debug/ {print $2}' $CONFIG_FILE | sed -E 's~^[ \t]+~~g' | sed -E 's~[ \t]+$~~g')
workers=$(awk -F "=" '/n_workers/ {print $2}' $CONFIG_FILE | sed -E 's~^[ \t]+~~g' | sed -E 's~[ \t]+$~~g')

# Parse out the API connection information
if echo "${conn_str}" | grep -iE "^https://" > /dev/null; then
    protocol="https"
else
    protocol="http"
fi

conn_no_protocol="$(echo "${conn_str}" | sed "s~^${protocol}://~~g")"
conn_user_host_port="$(echo "${conn_no_protocol}" | awk -F/ '{print $1}')"
if echo "${conn_user_host_port}" | grep "@"; then
    conn_user_pass="$(echo "${conn_user_host_port}" | awk -F@ '{print $1}')"
    conn_host_port="$(echo "${conn_user_host_port}" | awk -F@ '{print $2}')"
else
    conn_user_pass=""
    conn_host_port="${conn_user_host_port}"
fi

# Override configured host with 0.0.0.0 to ensure it listens on all interfaces, including lo
#conn_host="$(echo "${conn_host_port}" | awk -F: '{print $1}')"
conn_host=0.0.0.0 # Per discussion with Jeremy and Pat 2019.02.13
conn_port="$(echo "${conn_host_port}" | awk -F: '{print $2}')"

# Parse out the DB connection information

if echo "${db_str}" | grep -iE "^https://" > /dev/null; then
    db_protocol="https"
else
    db_protocol="http"
fi

db_no_protocol="$(echo "${db_str}" | sed "s~^${db_protocol}://~~g")"
db_user_host_port="$(echo "${db_no_protocol}" | awk -F/ '{print $1}')"
if echo "${db_user_host_port}" | grep "@"; then
    db_user_pass="$(echo "${db_user_host_port}" | awk -F@ '{print $1}')"
    db_host_port="$(echo "${db_user_host_port}" | awk -F@ '{print $2}')"
else
    db_user_pass=""
    db_host_port="${db_user_host_port}"
fi

db_host="$(echo "${db_host_port}" | awk -F: '{print $1}')"
db_port="$(echo "${db_host_port}" | awk -F: '{print $2}')"

STARTUP_ERROR=0

echo $(date +"%F %T")",000 INFO  (KML) startup - KML API Connection:     ${conn_str}"
echo $(date +"%F %T")",000 INFO  (KML) startup - Internal DB Connection: ${db_str}"

if [ -z "${conn_port}" ]; then
    echo $(date +"%F %T")",000 INFO  (KML) startup - ERROR: Unable to start AAW API because no port was specified for api_connection (api_connection=http://<host>:<port>)." >&2
    STARTUP_ERROR=1
fi

if [ -z "${db_host}" ] || [ -z "${db_port}" ]; then
    echo $(date +"%F %T")",000 INFO  (KML) startup - ERROR: Unable to start AAW API because database connection information was incomplete or missing (db_connection=http(s)://<host>:<port>)." >&2
    STARTUP_ERROR=1
fi

if [ "${STARTUP_ERROR}" != 0 ]; then
    exit 1
fi

# Add create system table and import models code here
echo $(date +"%F %T")",000 INFO  (KML) startup - Checking database connection"
if ! python3 ${KML_API_DIR}/hold_for_db.py; then
    echo $(date +"%F %T")",000 INFO  (KML) startup - ERROR: Database wait failure." >&2
    exit 1
fi
echo $(date +"%F %T")",000 INFO  (KML) startup - Starting KML setup script"
if ! python3 ${KML_API_DIR}/setup_tables_system.py; then
    echo $(date +"%F %T")",000 INFO  (KML) startup - ERROR: Kubernetes or Database issue.  Please review log output and retry." >&2
    exit 1
fi

# echo "Starting software components..."

# Start the State Manager (kmlsweep)
# echo "Starting state manager (kmlsweep)"
# nohup python3 ${KML_API_DIR}/kmlsweep.py > ${KML_API_DIR}/state_mgr_error.log 2>&1 &
# nohup python3 ${KML_API_DIR}/kmlsweep.py > ${KML_API_DIR}/logs/cluster_state.log 2>&1 &

echo $(date +"%F %T")",000 INFO  (KML) startup - Starting Uvicorn workers, setting root logger level to "$log_level

# Start the prefork application sever and mount the ASGI app
if [ $debug_mode == 'True' ]; then
    uvicorn $MODULE:app \
        --host "${conn_host}" \
        --port "${conn_port}" \
        --log-level "$log_level" \
        --debug
else
    # Run under Gunicorn:
    exec gunicorn $MODULE:app \
        -w "$workers" \
        -b "${conn_host}":"${conn_port}" \
        -k uvicorn.workers.UvicornWorker \
        --log-level "$log_level"

# Uvicorn only:
#     uvicorn $MODULE:app \
#         --host "${conn_host}" \
#         --port "${conn_port}" \
#         --log-level "$log_level" \
#         --log-config $KML_API_DIR"/uvicorn-log-config.json" \
#         --workers "$workers"
fi
