"""Kinetica Active Analytics Workbench async RESTful API dispacher."""

import os

from starlette.applications import Starlette
from starlette.middleware.cors import CORSMiddleware
from starlette.middleware.authentication import AuthenticationMiddleware

from kml.api.aux import (
    attach_log,
    BasicAuthBackend,
)

# Import all of the API component controllers
from kml.api import (
    controller_slipway,
    controller_ingest,
    controller_ingest_byoc,
    controller_featureset,
    controller_dataset,
    controller_kmlfile,
    controller_modelinstance,
    controller_deployment,
    controller_utils,
    controller_audit,
    controller_credential,
    controller_appstore,
    controller_rapidslab,
    controller_transform,
    controller_appstore
)

# Instantiate logger module
logger = attach_log(module=__name__, log_name='kml', debug=True)

# Define all API routes
#
# (
#     'path',
#     [methods],
#     controller
# )
#

endpoints = [

    # Quicklist endpoints
    (
        '/kml/ingest/quicklist',
        ['GET'],
        controller_utils.quick_list_ingest
    ),
    (
        '/kml/dataset/quicklist',
        ['GET'],
        controller_utils.quick_list_dataset
    ),
    (
        '/kml/featureset/quicklist',
        ['GET'],
        controller_utils.quick_list_featureset
    ),
    (
        '/kml/model/instance/quicklist',
        ['GET'],
        controller_utils.quick_list_model_instance
    ),
    (
        '/kml/model/deployment/quicklist',
        ['GET'],
        controller_utils.quick_list_model_deployment
    ),
    (
        '/kml/credential/quicklist',
        ['GET'],
        controller_utils.quick_list_credential
    ),

    # Model instantiation endpoints
    (
        '/kml/model/instance/create',
        ['POST', 'OPTIONS'],
        controller_modelinstance.model_inst_create
    ),
    (
        '/kml/model/rapids/instance/create',
        ['POST', 'OPTIONS'],
        controller_modelinstance.model_inst_create
    ),
    (
        '/kml/model/instance/{model_instance_id:int}/view',
        ['GET'],
        controller_modelinstance.model_inst_view
    ),
    (
        '/kml/model/instance/{model_instance_id:int}/clone',
        ['GET'],
        controller_modelinstance.model_inst_clone
    ),
    #  (
    #      '/kml/model/instance/{model_instance_id:int}/deploy',
    #      ['POST', 'OPTIONS'],
    #      controller_deployment.modelinstance_deploy
    #  ),
    (
        '/kml/model/instance/{model_instance_id:int}/deploy',
        ['POST', 'OPTIONS'],
        controller_deployment.modelinstance_deploy
    ),
    (
        '/kml/model/instance/{model_instance_id:int}/metrics/set',
        ['POST', 'OPTIONS'],
        controller_modelinstance.model_inst_push_metrics
    ),
    (
        '/kml/model/instance/{model_instance_id:int}/archive',
        ['GET'],
        controller_modelinstance.model_inst_archive
    ),
    # Appstore endpoints    
    (
        '/kml/introspect',
        ['POST'],
        controller_appstore.container_introspect
    ),    
    # Blackbox endpoints    
    (
        '/kml/model/deployment/{model_deployment_id:int}/stop',
        ['GET'],
        controller_deployment.model_dep_stop
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/start',
        ['GET'],
        controller_deployment.model_dep_start
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/complete',
        ['GET'],
        controller_deployment.model_deploy_complete
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/archive',
        ['GET'],
        controller_deployment.model_deploy_archive
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/view',
        ['GET'],
        controller_deployment.model_deploy_view
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/setstatus',
        ['POST', 'OPTIONS'],
        controller_deployment.model_deploy_setstatus
    ),
    (
        '/kml/model/deployment/{model_deployment_id:int}/infer',
        ['POST', 'OPTIONS'],
        controller_deployment.doinfer
    ),

    # Featureset endpoints
    (
        '/kml/featureset/create',
        ['POST', 'OPTIONS'],
        controller_featureset.featureset_create
    ),
    (
        '/kml/featureset/{featureset_id:int}/export',
        ['GET'],
        controller_featureset.featureset_export
    ),
    (
        '/kml/featureset/import',
        ['POST', 'OPTIONS'],
        controller_featureset.featureset_import
    ),
    (
        '/kml/featureset/{featureset_id:int}/refit',
        ['POST', 'OPTIONS'],
        controller_featureset.featureset_refit
    ),
    (
        '/kml/featureset/{featureset_id:int}/view',
        ['GET'],
        controller_featureset.featureset_view
    ),
    (
        '/kml/featureset/{featureset_id:int}/clone',
        ['GET'],
        controller_featureset.featureset_clone
    ),
    (
        '/kml/featureset/{featureset_id:int}/archive',
        ['GET'],
        controller_featureset.featureset_archive
    ),
    (
        '/kml/featureset/{featureset_id:int}/apply',
        ['POST', 'OPTIONS'],
        controller_transform.featureset_apply
    ),

    # Dataset endpoints
    (
        '/kml/dataset/testtrainsplit',
        ['POST', 'OPTIONS'],
        controller_dataset.dataset_testtrainsplit
    ),
    (
        '/kml/dataset/create',
        ['POST', 'OPTIONS'],
        controller_dataset.dataset_create
    ),
    (
        '/kml/dataset/{dataset_id:int}/view',
        ['GET'],
        controller_dataset.dataset_view
    ),
    (
        '/kml/dataset/{dataset_id:int}/clone',
        ['GET'],
        controller_dataset.dataset_clone
    ),
    (
        '/kml/dataset/{dataset_id:int}/preview',
        ['POST', 'OPTIONS'],
        controller_dataset.dataset_preview
    ),
    (
        '/kml/dataset/{dataset_id:int}/grab',
        ['GET'],
        controller_dataset.dataset_grab
    ),
    (
        '/kml/dataset/{dataset_id:int}/archive',
        ['GET'],
        controller_dataset.dataset_archive
    ),

    # Audit endpoints
    (
        '/kml/audit/model/{model_instance_id:int}/fields',
        ['GET'],
        controller_audit.get_audit_fields
    ),
    (
        '/kml/audit/model/{model_instance_id:int}/deployments',
        ['POST', 'OPTIONS'],
        controller_audit.get_dep_matches
    ),
    (
        '/kml/audit/{audit_id:int}/deployment/{model_deployment_id:int}/inferences',
        ['GET'],
        controller_audit.get_inferences
    ),
    (
        '/kml/audit/deployment/{model_deployment_id:int}/inference/{guid}',
        ['GET'],
        controller_audit.generate_audit_graph
    ),
    (
        '/kml/audit/{audit_id:int}/view',
        ['GET'],
        controller_audit.audit_view
    ),

    # RapidsLab endpoints
    (
        '/kml/jupyter/start',
        ['POST', 'OPTIONS'],
        controller_rapidslab.jupyter_start
    ),
    (
        '/kml/jupyter/stop',
        ['GET'],
        controller_rapidslab.jupyter_stop
    ),
    (
        '/kml/jupyter/status',
        ['GET'],
        controller_rapidslab.jupyter_status
    ),

    # General utilitiy endpoints
    (
        '/',
        ['GET'],
        controller_utils.index
    ),
    (
        '/kml/ping',
        ['GET'],
        controller_utils.ping
    ),
    (
        '/kml/latency/{n_records:int}',
        ['GET'],
        controller_utils.db_latency
    ),
    (
        '/kml/admin/events/register',
        ['POST'],
        controller_utils.register_events
    ),
    (
        '/kml/admin/events/retrieve',
        ['GET'],
        controller_utils.retrieve_events
    ),
    (
        '/kml/id/{my_id:int}/name/{my_name}/echo',
        ['POST', 'OPTIONS'],
        controller_utils.echo
    ),
    (
        '/kml/admin/cluster',
        ['GET'],
        controller_utils.mycluster
    ),
    (
        '/kml/admin/cluster/deployments',
        ['GET'],
        controller_utils.mycluster_deployments
    ),
    (
        '/kml/admin/cluster/familytree',
        ['GET'],
        controller_utils.mycluster_deployments_and_replicas
    ),
    (
        '/kml/admin/cluster/deployment/{model_deployment_id:int}/info',
        ['GET'],
        controller_utils.mycluster_deployment_info
    ),
    (
        '/kml/admin/cluster/deployment/{model_deployment_id:int}/logs',
        ['GET'],
        controller_utils.mycluster_deployment_logs
    ),
    (
        '/kml/admin/cluster/prepull',
        ['POST', 'OPTIONS'],
        controller_appstore.container_prepull
    ),
    (
        '/kml/admin/defaults/podresources',
        ['GET'],
        controller_utils.pod_resource_defaults
    ),

    # Credentials endpoints
    (
        '/kml/credential/create',
        ['POST', 'OPTIONS'],
        controller_credential.credential_create
    ),
    (
        '/kml/credential/{credential_id:int}/view',
        ['GET'],
        controller_credential.credential_view
    ),
    (
        '/kml/credential/{credential_id:int}/archive',
        ['GET'],
        controller_credential.credential_archive
    ),

    # File endpoints
    (
        '/kml/file/fname/{fname:str}/ftype/{ftype:str}/drop',
        ['POST', 'OPTIONS'],
        controller_kmlfile.accept_filedrop
    ),
    (
        '/kml/file/{file_id:int}/',
        ['GET'],
        controller_kmlfile.file_get
    ),
    (
        '/kml/file/list',
        ['GET'],
        controller_kmlfile.file_list
    ),
    (
        '/kml/file/{file_id:int}/view',
        ['GET'],
        controller_kmlfile.file_view
    ),

    # Data ingest endpoints
    (
        '/kml/ingest/create',
        ['POST', 'OPTIONS'],
        controller_ingest.ingest_create
    ),
        (
        '/kml/ingest/{ingest_id:int}/view',
        ['GET'],
        controller_ingest.ingest_view
    ),
    (
        '/kml/ingest/{ingest_id:int}/clone',
        ['GET'],
        controller_ingest.ingest_clone
    ),
    (
        '/kml/ingest/{ingest_id:int}/start',
        ['GET'],
        controller_ingest.ingest_start
    ),
    (
        '/kml/ingest/{ingest_id:int}/terminate',
        ['GET'],
        controller_ingest.ingest_terminate
    ),
    (
        '/kml/ingest/{ingest_id:int}/setstatus',
        ['POST', 'OPTIONS'],
        controller_ingest.ingest_setstatus
    ),
    (
        '/kml/ingest/{ingest_id:int}/archive',
        ['GET'],
        controller_ingest.ingest_archive
    ),
    (
        '/kml/ingest/status',
        ['GET'],
        controller_ingest.ingest_status
    ),

    # Container Slipway (simplified container direct runner)
    (
        '/kml/slipway/start',
        ['POST'],
        controller_slipway.direct_runner_start
    ),
    (
        '/kml/slipway/stop',
        ['POST'],
        controller_slipway.direct_runner_stop
    ),
    (
        '/kml/slipway/list',
        ['GET'],
        controller_slipway.direct_runner_list
    ),
    (
        '/kml/slipway/purge',
        ['GET'],
        controller_slipway.direct_runner_purge
    ),

]

# Instantiate the ASGI object
app = Starlette()

# Add the API routes
for (routepath, routemethod, routecontroller) in endpoints:
    app.add_route(
        path=routepath,
        methods=routemethod,
        route=routecontroller
    )

# Add Cross-Origin support
app.add_middleware(
    CORSMiddleware,
    allow_origins=['*'],
    allow_methods=['HEAD', 'GET', 'POST', 'OPTIONS'],
    allow_headers=['*'],
    expose_headers=['*']
)

# Add Basic Auth support
app.add_middleware(
    AuthenticationMiddleware,
    backend=BasicAuthBackend()
)

logger.info(f'Kinetica ML API worker started [{os.getpid()}]')
