"""KML kmlfile endpoint controller."""

import os
import traceback
import tempfile

from kml.api.aux import attach_log, Responder
from kml.api.orm import Entity, KMLFile


# Logging module
logger = attach_log(module=__name__, log_name='kml', debug=True)

def file_get(request):
    from starlette.responses import Response

    # Handle both internal and http function call cases
    if type(request) is int:
        file_id = request
    else:
        file_id = request.path_params.get('file_id')

    response = Responder()
    try:
        # TODO: Do this all *WITHOUT* saving it temporarily (just straight pass-the-potato)
        _, tempfilename = tempfile.mkstemp()
        if not KMLFile.exists(file_id):
            response.response["found"]=False
            logger.debug(f"Could not get non-existent file with file_id {file_id}")
            response.failed(msg=f"File with ID {file_id} not found", code=1040)
            return response.emit(404)

        f_loaded = KMLFile.load(file_id)
        f_loaded.save_file(tempfilename)

        with open(tempfilename,"rb") as f:
            data = f.read()
            #TODO: Remove entry
            entry = {'name': 'adsf'}
            FILE_RETURN_HEADER={}
            FILE_RETURN_HEADER['Access-Control-Allow-Methods']='HEAD, GET'
            FILE_RETURN_HEADER['Access-Control-Allow-Origin']='*'
            FILE_RETURN_HEADER['Content-Type']='application/tar+gzip'
            FILE_RETURN_HEADER['Content-Disposition']='attachment; filename="'+f_loaded.file_name+'"'
            return Response(data, headers=FILE_RETURN_HEADER)
    except Exception as e:
        logger.error(e)
        logger.debug(traceback.format_exc())
        response.failed(msg=e, code=1040)
        return response.emit(509)

def file_list(request):
    response = Responder()
    response.response["found"]=False
    try:
        inventory = KMLFile.universe()
        response.response["count"]=len(inventory)
        if len(inventory)>0:
            response.response["found"]=True

        response.response["inventory"]=[inventory[k] for k in inventory.keys()]
        response.succeeded()
        return response.emit(200)
    except Exception as e:
        logger.error(e)
        logger.debug(traceback.format_exc())
        response.failed(msg=e, code=1040)
        return response.emit(509)

def file_view(request):
    response = Responder()
    response.response["found"]=False
    try:
        file_id = request.path_params.get('file_id')
        file_record = KMLFile.load(file_id)
        response.response['item'] = file_record.serialize()
        response.response["found"]=False
        response.succeeded()
        return response.emit(201)
    except Exception as e:
        logger.error(e)
        logger.debug(traceback.format_exc())
        response.failed(msg=e, code=1040)
        return response.emit(509)

async def accept_filedrop(request):
    response = Responder()
    response.response["persisted"] = False
    try:
        # grab what came thru the post and stash it into a temp file
        body = await request.body()
        # TODO: Although for multipart the following is probably preferable:
        # multi = await request.form()

        if request.path_params.get('fname').lower() == 'none':
            fname = None
        else:
            fname = request.path_params['fname']

        if request.path_params.get('ftype').lower() == 'none':
            ftype = None
        else:
            ftype = request.path_params['ftype']

        logger.debug(f"Processing file {fname} with type {ftype}")

        _, filename = tempfile.mkstemp()
        if ftype and type(ftype) == str:
            filename = '.'.join([str(filename), ftype.lower()])
        fd = os.open(filename, os.O_WRONLY | os.O_CREAT | os.O_TRUNC)
        os.write(fd, body)
        os.close(fd)
        # store bytes blob into DB
        f0 = KMLFile.create(the_file_path=filename, file_name=filename, file_desc=fname)
        response.response["created"]=f0.serialize()
        response.response["size"]=os.stat(filename).st_size
        response.response["persisted"] = True
        response.succeeded()
        return response.emit(201)
    except Exception as e:
        logger.error(e)
        logger.debug(traceback.format_exc())
        response.failed(msg=e, code=1040)
        return response.emit(509)
