def gen_slug_hash():
    """Eight character hash seeded with microsec timestamp."""

    import time
    from hashlib import sha256

    return sha256(str(time.time()).encode('utf-8')).hexdigest()[:8]


# Deployment Manager
def get_cluster_name_dep_mgr(in_deployment_id):
    return f'kml-mgr-{in_deployment_id:05}'


# Input manager
def get_cluster_name_input_mgr(in_deployment_id):
    return f'kml-inp-{in_deployment_id:05}'


# Service
def get_cluster_name_service(tag):
    return f'kml-svc-{tag:05}' if isinstance(tag, int) else f'kml-svc-{tag}'


# Ingress definition
def get_cluster_name_ingress(tag):
    return f'kml-igr-{tag:05}' if isinstance(tag, int) else f'kml-igr-{tag}'


# Default on-demand ingress path slug
def get_ondemand_default_slug(in_deployment_id):
    return f'dep_{in_deployment_id}_{gen_slug_hash()}'


# Default JupyterLab ingress path slug
def get_jupyter_default_slug(profile):
    return f'lab_{profile}_{gen_slug_hash()}'


# BlackBox Worker
def get_cluster_name_deployment_blackbox(in_deployment_id):
    return f'kml-bbx-{in_deployment_id:05}'


# Ingest
def get_cluster_name_ingest(in_ingest_id):
    return f'kml-ing-{in_ingest_id:05}'


# JupyterLab
def get_cluster_name_jupyter(profile):
    return f'kml-jpt-{profile}'


def get_audit_table_names(in_deployment_id):
    audit_in = f"dep-{in_deployment_id:05}-inp"
    audit_out = f"dep-{in_deployment_id:05}-out"
    return audit_in, audit_out


def get_entity_id_from_k8s_name(k8_obj_name):
    return int(k8_obj_name[-5:])


def get_container_type(k8_obj_name):

    if len(k8_obj_name) < 8:
        return "UNKNOWN"

    ctnr_type_triad = k8_obj_name[4:7]

    if ctnr_type_triad == "mgr":
        return "DEPLOYMENT_MANAGER"

    if ctnr_type_triad == "inp":
        return "DATA_INPUT_MANAGER"

    if ctnr_type_triad == "bbx":
        return "BLACK_BOX_WORKER"

    if ctnr_type_triad == "ing":
        return "INGEST"

    if ctnr_type_triad == "jpt":
        return "JUPYTERLAB"

    if ctnr_type_triad == "svc":
        return "SERVICE"

    if ctnr_type_triad == "igr":
        return "INGRESS"

    return "UNKNOWN"


# Credential Handle on k8s
def get_k8s_secret_handle(in_credential_id):
    return f'kml-k8s-secrets-handle-{in_credential_id:05}'
