import json


def validate_resp_metadata(resp):
    """Check metadata in base response."""

    assert resp.get('success')
    assert resp.get('response')
    assert resp.get('processing_time') > 0

    return True


def validate_resp_metadata_fail(resp, fail_msg):
    """Check metadata in base failure response."""

    assert not resp.get('success')
    assert resp.get('processing_time') > 0
    assert resp.get('error')
    assert 'message' in resp['error']
    assert fail_msg in resp['error']['message']

    return True


def validate_manifest_bbox(resp):
    """Check manifest for Kinetica BlackBox Model Containers."""

    assert resp['response'].get('name')
    assert resp['response'].get('manifest')
    manifest = resp['response']['manifest']

    assert manifest.get('name')
    assert 'desc' in manifest
    assert manifest.get('src_uri')
    assert manifest.get('is_available')
    assert manifest.get('type') == 'BLACKBOX'

    assert manifest.get('functions')
    assert len(manifest['functions']) > 0
    for func in manifest['functions']:
        assert func.get('name'), json.dumps(func)
        assert 'desc' in func, json.dumps(func)
        assert func.get('bb_module' ), json.dumps(func)
        assert func.get('bb_function'), json.dumps(func)
        assert 'input_record_type' in func, json.dumps(func)
        assert 'output_record_type' in func, json.dumps(func)
        assert 'env_vars' in func, json.dumps(func)

    return True


def validate_manifest_byoc(resp):
    """Check manifest for Kinetica Custom Ingest Containers."""

    assert resp['response'].get('name')
    assert resp['response'].get('manifest')
    manifest = resp['response']['manifest']

    assert manifest.get('name')
    assert 'desc' in manifest
    assert manifest.get('src_uri')
    assert 'env_vars' in manifest
    assert manifest.get('type') == 'INGEST'
    assert manifest.get('ingest_mode') in ['CONTINUOUS', 'BATCH']

    return True


def validate_credential_create(resp):
    """Check manifest for credential creation."""

    assert resp['response'].get('created')
    assert resp['response']['created'].get('entity')
    entity = resp['response']['created']['entity']

    assert entity.get('entity_id') > 0
    assert entity.get('entity_type') == 'CREDENTIAL'
    assert entity.get('entity_state') == 'CREATED'

    assert resp['response']['created'].get('credential_type') in ['DOCKER']
    assert resp['response']['created'].get('credential_config')
    config = resp['response']['created']['credential_config']
    assert resp['response']['created'].get('k8s_secrets_handle')
    assert resp['response']['created']\
        ['k8s_secrets_handle'].startswith('kml-k8s-secrets-handle')
    assert str(entity['entity_id']) in resp['response']['created']['k8s_secrets_handle']

    assert config.get('registry_uri') in ['https://index.docker.io/v1/']
    assert config.get('user')
    assert config.get('email')
    assert config.get('password')

    return True


